body {
    padding-top: 70px
}
.gnc-superaccount[data-bs-toggle].collapsed:before {
    content: "\f282";
    color:  #444444;
    position: relative;
    left: -4px;
}
.gnc-superaccount[data-bs-toggle]:not(.collapsed):before {
    font-weight: 900;
    content: "\f286";
    color:  #444444;
    position: relative;
    left: -4px;
}

.gnc-account-group {
    padding-top: 0rem;
    padding-bottom: 0rem;
    padding-right: 0rem;

    border-left-width: 0px;
    border-right-width: 0px;
    border-bottom-width: 0px;
    border-top-width: 0px;
}

.gnc-account-group + .gnc-superaccount,
.gnc-account-group + .gnc-leafaccount {
    border-top-width: 1px;
}

.gnc-account-nav {
    --bs-breadcrumb-divider: ':';
}

a {
    text-decoration: none;
}

.no-text-break {
    word-break: normal;
    word-wrap: normal;
}

.scrolled-down{
  transform:translateY(-100%); transition: all 0.3s ease-in-out;
}
.scrolled-up{
  transform:translateY(0); transition: all 0.3s ease-in-out;
}

.hidden {
    display: none
}

.selectize-control.was-validated > .selectize-input {
    padding-right: calc(1.5em + .75rem);
    background-repeat: no-repeat;
    background-position: right 2.3em center;
    background-size: calc(.75em + .375rem) calc(.75em + .375rem);
}

.selectize-control.is-valid > .selectize-input {
    border-color: var(--bs-success);
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23198754' d='M2.3 6.73.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e");
}

.selectize-control.is-invalid > .selectize-input {
    border-color: var(--bs-danger);
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23dc3545'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23dc3545' stroke='none'/%3e%3c/svg%3e");
}

// See: https://stackoverflow.com/questions/23802562/bootstrap-btn-group-with-disabled-button-tooltip-wrapper-mess-with-btn-order?rq=1
.tooltip-wrapper {
    display: inline-block;
    float: left;
}
.tooltip-wrapper .btn {
    border-radius: 0;
}
.tooltip-wrapper:first-child .btn {
    margin-right: calc(var(--bs-border-width) * -1);
    border-radius: var(--bs-btn-border-radius) 0 0 var(--bs-btn-border-radius);
}
.tooltip-wrapper:last-child .btn {
    margin-left: calc(var(--bs-border-width) * -1);
    border-radius: 0 var(--bs-btn-border-radius) var(--bs-btn-border-radius) 0;
}
